@@ -167,7 +167,9 @@ class DB:
 class API:
     section = 'api'
     TITLE = config.get(section, 'title', fallback='TensorHive API')
+    URL_SCHEMA = config.get(section, 'url_schema', fallback='http')
     URL_HOSTNAME = config.get(section, 'url_hostname', fallback='0.0.0.0')
+    URL_PORT = config.get(section, 'url_port', fallback='1111')
     URL_PREFIX = config.get(section, 'url_prefix', fallback='api')
     SPEC_FILE = config.get(section, 'spec_file', fallback='api_specification.yml')
     IMPL_LOCATION = config.get(section, 'impl_location', fallback='tensorhive.api.controllers')
@@ -204,16 +206,19 @@ class MONITORING_SERVICE:
 
 class PROTECTION_SERVICE:
     section = 'protection_service'
-    ENABLED = config.getboolean(section, 'enabled', fallback=True)
+    LEVEL = config.getint(section, 'level', fallback=1)
     UPDATE_INTERVAL = config.getfloat(section, 'update_interval', fallback=2.0)
     NOTIFY_ON_PTY = config.getboolean(section, 'notify_on_pty', fallback=True)
     NOTIFY_VIA_EMAIL = config.getboolean(section, 'notify_via_email', fallback=False)
+    KILL_PROCESSES = config.getint(section, 'kill_processes', fallback=0)
 
 
 class MAILBOT:
     mailbot_config = ConfigLoader.load(CONFIG_FILES.MAILBOT_CONFIG_PATH, displayed_title='mailbot')
     section = 'general'
     INTERVAL = mailbot_config.getfloat(section, 'interval', fallback=10.0)
+    MAX_EMAILS_PER_PROTECTION_INTERVAL = mailbot_config.getint(section,
+                                                               'max_emails_per_protection_interval', fallback=50)
     NOTIFY_INTRUDER = mailbot_config.getboolean(section, 'notify_intruder', fallback=True)
     NOTIFY_ADMIN = mailbot_config.getboolean(section, 'notify_admin', fallback=False)
     ADMIN_EMAIL = mailbot_config.get(section, 'admin_email', fallback=None)
@@ -246,11 +251,12 @@ class USAGE_LOGGING_SERVICE:
     LOG_CLEANUP_ACTION = config.getint(section, 'log_cleanup_action', fallback=2)
 
 
-class TASK_SCHEDULING_SERVICE:
-    section = 'task_scheduling_service'
+class JOB_SCHEDULING_SERVICE:
+    section = 'job_scheduling_service'
     ENABLED = config.getboolean(section, 'enabled', fallback=True)
     UPDATE_INTERVAL = config.getfloat(section, 'update_interval', fallback=30.0)
     STOP_TERMINATION_ATTEMPTS_AFTER = config.getfloat(section, 'stop_termination_attempts_after_mins', fallback=5.0)
+    SCHEDULE_QUEUED_JOBS_WHEN_FREE_MINS = config.getint(section, "schedule_queued_jobs_when_free_mins", fallback=30)
 
 
 class AUTH:
