@@ -7,7 +7,7 @@ from tensorhive.core.services.Service import Service
 from typing import List, Dict
 from tensorhive.core.utils.decorators import override
 from tensorhive.config import (SSH, MONITORING_SERVICE, PROTECTION_SERVICE, USAGE_LOGGING_SERVICE,
-                               TASK_SCHEDULING_SERVICE)
+                               JOB_SCHEDULING_SERVICE)
 from tensorhive.api.APIServer import APIServer
 from tensorhive.core.utils.StoppableThread import StoppableThread
 from tensorhive.core.utils.exceptions import ConfigurationException
@@ -17,10 +17,13 @@ from tensorhive.core.monitors.CPUMonitor import CPUMonitor
 from tensorhive.core.services.MonitoringService import MonitoringService
 from tensorhive.core.services.ProtectionService import ProtectionService
 from tensorhive.core.services.UsageLoggingService import UsageLoggingService
-from tensorhive.core.services.TaskSchedulingService import TaskSchedulingService
+from tensorhive.core.services.JobSchedulingService import JobSchedulingService
 from tensorhive.core.violation_handlers.ProtectionHandler import ProtectionHandler
 from tensorhive.core.violation_handlers.MessageSendingBehaviour import MessageSendingBehaviour
 from tensorhive.core.violation_handlers.EmailSendingBehaviour import EmailSendingBehaviour
+from tensorhive.core.violation_handlers.UserProcessKillingBehaviour import UserProcessKillingBehaviour
+from tensorhive.core.violation_handlers.SudoProcessKillingBehaviour import SudoProcessKillingBehaviour
+from tensorhive.core.scheduling import GreedyScheduler
 from tensorhive.core import ssh
 from pathlib import PosixPath
 import logging
@@ -76,7 +79,14 @@ class TensorHiveManager(metaclass=Singleton):
             # TODO Add more monitors here
             monitoring_service = MonitoringService(monitors=monitors, interval=MONITORING_SERVICE.UPDATE_INTERVAL)
             services.append(monitoring_service)
-        if PROTECTION_SERVICE.ENABLED:
+        if JOB_SCHEDULING_SERVICE.ENABLED:
+            job_scheduling_service = JobSchedulingService(
+                interval=JOB_SCHEDULING_SERVICE.UPDATE_INTERVAL,
+                stop_attempts_after=JOB_SCHEDULING_SERVICE.STOP_TERMINATION_ATTEMPTS_AFTER)
+            # TODO: scheduler configuration when more schedulers are available
+            job_scheduling_service.inject(GreedyScheduler())
+            services.append(job_scheduling_service)
+        if PROTECTION_SERVICE.LEVEL:
             violation_handlers = []
             if PROTECTION_SERVICE.NOTIFY_ON_PTY:
                 message_sending_handler = ProtectionHandler(behaviour=MessageSendingBehaviour())
@@ -84,17 +94,19 @@ class TensorHiveManager(metaclass=Singleton):
             if PROTECTION_SERVICE.NOTIFY_VIA_EMAIL:
                 email_sending_handler = ProtectionHandler(behaviour=EmailSendingBehaviour())
                 violation_handlers.append(email_sending_handler)
+            if PROTECTION_SERVICE.KILL_PROCESSES:
+                if PROTECTION_SERVICE.KILL_PROCESSES > 1:
+                    process_killing_handler = ProtectionHandler(behaviour=SudoProcessKillingBehaviour())
+                else:
+                    process_killing_handler = ProtectionHandler(behaviour=UserProcessKillingBehaviour())
+                violation_handlers.append(process_killing_handler)
             protection_service = ProtectionService(
-                handlers=violation_handlers, interval=PROTECTION_SERVICE.UPDATE_INTERVAL)
+                handlers=violation_handlers, interval=PROTECTION_SERVICE.UPDATE_INTERVAL,
+                strict_reservations=PROTECTION_SERVICE.LEVEL > 1)
             services.append(protection_service)
         if USAGE_LOGGING_SERVICE.ENABLED:
             usage_logging_service = UsageLoggingService(interval=USAGE_LOGGING_SERVICE.UPDATE_INTERVAL)
             services.append(usage_logging_service)
-        if TASK_SCHEDULING_SERVICE:
-            task_scheduling_service = TaskSchedulingService(
-                interval=TASK_SCHEDULING_SERVICE.UPDATE_INTERVAL,
-                stop_attempts_after=TASK_SCHEDULING_SERVICE.STOP_TERMINATION_ATTEMPTS_AFTER)
-            services.append(task_scheduling_service)
         return services
 
     def configure_services_from_config(self):
