@@ -1,16 +1,15 @@
 from tensorhive.core.utils.decorators import memoize, timeit
 from tensorhive.config import SSH
 from pssh.clients.native import ParallelSSHClient
-from typing import Optional, Dict, Tuple
+from pssh.exceptions import AuthenticationException
+from typing import Optional, Dict, Tuple, Generator, List
 from paramiko.rsakey import RSAKey
-import functools
 from pathlib import PosixPath
 import pssh
 import logging
 log = logging.getLogger(__name__)
 
-__author__ = '@micmarty'
-__all__ = ['build_dedicated_config_for', 'get_client', 'run_command', 'get_stdout', 'succeeded']
+__author__ = '@micmarty, @roscisz'
 """
 This module provides a universal and stateless API for SSH-related tasks.
 
@@ -37,10 +36,12 @@ def build_dedicated_config_for(host: Hostname, user: Username) -> Tuple[HostsCon
     valid `config` and `pconfig` parameter to `get_client()` function.
     """
     assert host and user, 'Arguments must not be None!'
+    assert host in SSH.AVAILABLE_NODES
     hosts_config = {
         host: {
             'user': user,
-            'pkey': SSH.KEY_FILE
+            'pkey': SSH.KEY_FILE,
+            'port': SSH.AVAILABLE_NODES[host]['port']
         }
     }
     # Read config extracted from hosts_config.ini (proxy is common for all hosts)
@@ -112,6 +113,9 @@ def get_stdout(host: Hostname, output: pssh.output.HostOutput) -> Optional[str]:
     except (TypeError, ):
         log.warning('Could not extract stdout for {}: {}'.format(host, output))
         return None
+    except AuthenticationException:
+        log.warning('Could not authenticate SSH connection for {}: {}'.format(host, output))
+        return None
     except Exception as e:
         log.critical(e)
         # Base for all pssh exceptions: https://github.com/ParallelSSH/parallel-ssh/blob/master/pssh/exceptions.py
@@ -139,3 +143,36 @@ def init_ssh_key(path: PosixPath):
         key = generate_cert(path)
         log.info('[⚙] Generated SSH key in {}'.format(path))
     return key
+
+
+def node_tty_sessions(connection) -> List[Dict]:
+    '''Executes shell command in order to fetch all active terminal sessions'''
+    command = 'who'
+    output = connection.run_command(command)
+
+    # FIXME Assumes that only one node is in connection
+    for _, host_out in output.items():
+        result = _parse_who_output(host_out.stdout)
+    return result
+
+
+def _parse_who_output(stdout: Generator) -> List[Dict]:
+    '''
+    Transforms command output into a dictionary
+    Assumes command was: 'who | grep <username>'
+    '''
+    stdout_lines = list(stdout)  # type: List[str]
+
+    # Empty stdout
+    if stdout_lines is None:
+        return None
+
+    def as_dict(line):
+        columns = line.split()
+        return {
+            # I wanted it to be more explicit and flexible (even if it could be done better)
+            'USER': columns[0],
+            'TTY': columns[1]
+        }
+
+    return [as_dict(line) for line in stdout_lines]
